/*******************************************************************************
 * {COPYRIGHT-TOP} *** * IBM Confidential * OCO Source Materials 5725-E59
 * 
 * (c) Copyright IBM Corp. 2012 All Rights Reserved.
 * 
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 ******************************************************************************/
define(
        [ "dojo/_base/declare", "dijit/_Widget",
                "dojo/i18n!wodm/decisions/nls/manageDecisionsController",
                "dojo/on", "dojo/_base/xhr", "dojo/_base/lang", "dojo/string",
                "dojo/dom-construct", "dojo/dom-style", "dojo/dom-class",
                "dojo/aspect", "ibm/ccd/dialogs/CommonDialog",
                "dijit/form/Button", "wodm/decisions/ErrorWidget" ],
        function(declare, _Widget, i18nResources, on, xhr, lang, dojoString,
                domConstruct, domStyle, domClass, aspect, CommonDialog, Button,
                ErrorWidget) {

            /**
             * The module wodm.decisions.EditRuleDialog provides a Widget that
             * allows the user to edit a rule using the WODM Syntactic Editor.
             */

            var EditRuleDialog = declare(
                    "wodm.decisions.EditRuleDialog",
                    [ _Widget ],
                    {

                        /**
                         * WODM common configuration properties.
                         */
                        _config : dojo.config.wodm,

                        baseClass : "wodmDecisionsEditRuleDialog",

                        /**
                         * URLS for various xhr requests/navigation links for
                         * the WODM Edit Rule Dialog Widget.
                         */
                        _urls : {

                            EDIT_RULE : "/user/wodm/jsp/decisions/editRule.jsp",
                            SAVE_RULE : "wodmSaveRule.wpc",
                            CANCEL_RULE_EDITING : "wodmCancelRuleEditing.wpc"

                        },

                        /**
                         * Object containing properties for buttons that will be
                         * added to the Add Rule Dialog button bar.
                         */
                        _buttonPropsMap : null,

                        /**
                         * Name of the rule project associated with the rule
                         * being edited in this EditRuleDialog Widget.
                         */
                        ruleProjectName : null,

                        /**
                         * Name of the rule being edited in this EditRuleDialog
                         * Widget.
                         */
                        ruleName : null,

                        /**
                         * Dom node for a containing div for the ErrorWidget.
                         */
                        _errorContainer : null,

                        /**
                         * Object containing reference to a ErrorWidget used for
                         * displaying errors in the EditRuleDialog Widget.
                         */
                        _errorWidget : null,

                        /**
                         * Object containing reference to the underlying
                         * CommonDialog widget.
                         */
                        _commonDialog : null,

                        /**
                         * Array of connection handles. Used to store dojo
                         * connections that can be disconnected on destroy.
                         */
                        _connectionHandles : [],

                        postMixInProperties : function() {

                            this.inherited(arguments);
                            this._i18nResources = i18nResources;

                        },

                        postCreate : function() {

                            this.inherited(arguments);
                            this._initializeButtonPropsMap();

                        },

                        /**
                         * Returns a url to be used for the value of the 'src'
                         * attribute of the iframe used to host the WODM
                         * Syntactice Editor.
                         */
                        _getFrameSrcUrl : function() {

                            return this._urls.EDIT_RULE + "?ruleProjectName="
                                    + this.ruleProjectName + "&ruleName="
                                    + this.ruleName + "&containerType="
                                    + this._config.containerType + "&entryPk="
                                    + this._config.entryPk;

                        },

                        /**
                         * Returns title for the Dialog.
                         */
                        _getTitle : function() {

                            return dojoString.substitute(
                                    this._i18nResources.EDIT_RULE_DIALOG_TITLE,
                                    [ this.ruleName ]);

                        },

                        /**
                         * Creates an instance of the CommonDialog widget.
                         */
                        _createDialog : function() {

                            this._commonDialog = new CommonDialog({

                                title : this._getTitle(),
                                showButtonBar : true,
                                showOkButton : false,
                                showCancelButton : true,
                                style : "width: 90%"

                            });

                            this._createDialogContent();
                            this.createDialogButtons();
                            aspect
                                    .after(
                                            this._commonDialog,
                                            "onCancel",
                                            lang
                                                    .hitch(
                                                            this,
                                                            this._sendCancelRuleEditingRequest));

                        },

                        /**
                         * Creates the main content of the EditRuleDialog.
                         */
                        _createDialogContent : function() {

                            var editRuleContentContainer, errorContainer, frameContainer, url, frameHtml, frameHeight, frameId;
                            url = this._getFrameSrcUrl();

                            editRuleContentContainer = domConstruct.create(
                                    "div", {
                                        id : "editRuleContentContainer"
                                    });

                            errorContainer = domConstruct.create("div", {
                                id : "editRuleErrorContainer"
                            });
                            this._errorContainer = errorContainer;

                            frameContainer = domConstruct.create("div", {
                                id : "editRuleFrameContainer"
                            });
                            editRuleContentContainer
                                    .appendChild(errorContainer);
                            editRuleContentContainer
                                    .appendChild(frameContainer);

                            frameHeight = document.body.offsetHeight - 100;

                            domStyle.set(editRuleContentContainer, "height",
                                    frameHeight);

                            frameHeight = frameHeight > 450? frameHeight : 450; // Force vertical scrollbar if too short
                            frameId = "editRuleFrameContainerFrame";
                            frameHtml = "<iframe  name='"
                                    + frameId
                                    + "' id='"
                                    + frameId
                                    + "' src='"
                                    + url
                                    + "' style='height:"
                                    + frameHeight
                                    + "px;' class='iLogCustomTabFrame'></iframe>";

                            frameContainer.innerHTML = frameHtml;
                            domConstruct.place(editRuleContentContainer,
                                    this._commonDialog.contentNode, "only");

                        },

                        /**
                         * Initializes the properties of different buttons used
                         * in the Button Bar for the Edit Rule Dialog. This
                         * includes definition as well as handlers for each
                         * button. The properties will be available in the
                         * _buttonPropsMap after initialization.
                         */
                        _initializeButtonPropsMap : function() {

                            this._buttonPropsMap = {

                                "wodmEditRuleDialog.save" : {

                                    definition : {

                                        id : "wodmEditRuleDialog.save",
                                        label : this._i18nResources.EDIT_RULE_DIALOG_SAVE_BUTTON_LABEL,
                                        title : this._i18nResources.EDIT_RULE_DIALOG_SAVE_BUTTON_TOOLTIP
                                    },
                                    handlers : {

                                        click : lang.hitch(this,
                                                this._sendSaveRuleRequest)

                                    }

                                }

                            }

                        },

                        /**
                         * Creates Button widgets for the Button Bar in the Edit
                         * Rule Dialog Widget.
                         */
                        createDialogButtons : function() {

                            this
                                    ._addButton(this._buttonPropsMap["wodmEditRuleDialog.save"]);

                        },

                        /**
                         * Creates a new dijit.form.Button object based on the
                         * given properties object and adds it to the dialog
                         * button bar.
                         * 
                         * @param buttonProps
                         *            {Object} an object containing properties
                         *            of the button to be created. This includes
                         *            definition as well as handlers.
                         */
                        _addButton : function(buttonProps) {

                            var button, connectionHandle;
                            button = new Button(buttonProps.definition);
                            connectionHandle = on(button, "click",
                                    buttonProps.handlers.click);
                            this._connectionHandles.push(connectionHandle);
                            this._commonDialog.buttonBar.insertBefore(
                                    button.domNode,
                                    this._commonDialog.cancelButton.domNode);

                        },

                        /**
                         * Displays the AddRuleDialog Widget.
                         */
                        show : function() {

                            if (!this._commonDialog) {
                                this._createDialog();
                            } else {

                                this._commonDialog.set("title", this
                                        ._getTitle());

                                this._destroyErrorWidget();
                                this._createDialogContent();
                            }

                            this._commonDialog.show();

                        },

                        /**
                         * Hides the AddRuleDialog Widget.
                         */
                        hide : function() {
                            this._commonDialog.hide();
                        },

                        _destroyErrorWidget : function() {
                            if (this._errorWidget) {
                                domConstruct.destroy(this._errorWidget.domNode);
                                this._errorWidget = null;
                            }

                        },

                        /**
                         * Hook for the set method to work for the ruleName
                         * property.
                         * 
                         * @param value
                         *            {String} name of the rule being edited
                         */
                        _setRuleNameAttr : function(value) {

                            this.ruleName = value;
                        },

                        /**
                         * Hook for the set method to work for the
                         * ruleProjectName property.
                         * 
                         * @param value
                         *            {String} name of the rule project the rule
                         *            being edited belongs to
                         */
                        _setRuleProjectNameAttr : function(value) {

                            this.ruleProjectName = value;

                        },

                        /**
                         * Destroys the error widget if it exists.
                         */
                        _destroyErrorWidget : function() {
                            if (this._errorWidget) {
                                domConstruct.destroy(this._errorWidget.domNode);
                                this._errorWidget = null;
                            }

                        },

                        /**
                         * Creates and displays an instance of ErrorWidget
                         * displaying the errors provided.
                         * 
                         * @error {Array} an array of Strings of error messages
                         *        to display
                         */
                        _displayErrors : function(errors) {

                            this._destroyErrorWidget();
                            var errorWidget = new ErrorWidget({
                                errors : errors
                            });
                            this._errorWidget = errorWidget;
                            this._errorContainer
                                    .appendChild(errorWidget.domNode);

                        },

                        /**
                         * Sends a xhr get request to the server and invokes the
                         * appropriate given callback method on response.
                         * 
                         * @param url
                         *            {String} url to send the get request to
                         * @param onLoad
                         *            {Function} function to invoke on response
                         *            received from the server
                         * @param onError
                         *            {Function} function to invoke on error
                         */
                        _sendRequest : function(url, onLoad, onError) {

                            var reqObj = {

                                url : url,
                                handleAs : "json",
                                content : {
                                    ruleProjectName : this.ruleProjectName,
                                    ruleName : this.ruleName,
                                    containerType: this._config.containerType,
                                    entryPk: this._config.entryPk                                    
                                    

                                },
                                sync : true,
                                load : onLoad,
                                error : lang.hitch(this, function(error) {
                                    var errorObj = {
                                        error : error
                                    }
                                    onError(errorObj);
                                })

                            }

                            setTimeout(function() {

                                xhr.get(reqObj);
                            }, 1);

                        },

                        /**
                         * Sends a xhr request to the server to save the rule
                         * displayed in the EditRuleDialog Widget.
                         */
                        _sendSaveRuleRequest : function() {

                            var responseHandler = lang.hitch(this,
                                    this._handleSaveRuleRequestResponse);
                            this._sendRequest(this._urls.SAVE_RULE,
                                    responseHandler, responseHandler);

                        },

                        /**
                         * Response handler for the save rule request.
                         */
                        _handleSaveRuleRequestResponse : function(response) {

                            if (response.error) {

                                this._handleResponseError(response.error);
                                return;
                            }

                            this.onSuccessfulSave(response.rule);

                        },

                        /**
                         * Sends a xhr request to the server to cancel editing
                         * for the rule displayed in the EditRuleDialog Widget.
                         */
                        _sendCancelRuleEditingRequest : function() {
                            var _this = this;
                            var responseHandler = function(response) {
                            	
                                if (response.result) {
                                    _this.onSuccessfulCancel();
                                }
                            }
                            this._sendRequest(this._urls.CANCEL_RULE_EDITING,
                                    responseHandler, lang.hitch(this,
                                            this._handleResponseError));

                        },

                        /**
                         * Response handler for errors. Displays the errors
                         * extracted from the given object and displays them
                         * using an ErrorWidget.
                         */
                        _handleResponseError : function(error) {

                            var errors;
                            if (error.errorMessage) {
                                errors = [ error.errorMessage ];
                            } else if (error.errorMessages) {

                                errors = error.errorMessages;
                            }
                            this._displayErrors(errors);

                        },

                        /**
                         * Invoked on a successful save of the edited rule.
                         * Users of this Widget should connect to this method to
                         * receive notification about a successful save
                         * operation.
                         * 
                         * @param rule
                         *            {Object} json object representing the rule
                         *            that was edited. This does not contain the
                         *            entire rule representation. Note: This
                         *            object only contains fields LastUsedBy and
                         *            LastChangedOn
                         * 
                         */
                        onSuccessfulSave : function(rule) {

                        },

                        /**
                         * Invoked on a successful cancel of the edited rule.
                         * Users of this Widget should connect to this method to
                         * handle any post processing.
                         */
                        onSuccessfulCancel: function() {
                        	
                        },
                        
                        /**
                         * Widget life cycle method. This function is called by
                         * the dojo framework when destroy is invoked on a
                         * widget. This method should not be invoked directly,
                         * instead invoke the destroy method.
                         */
                        uninitialize : function() {

                            this._destroyErrorWidget();
                            baseArray.forEach(this._connectionHandles,
                                    function(handle) {

                                        handle.remove();

                                    });
                            this._commonDialog.destroy();

                        }

                    });
            return EditRuleDialog;

        });