/*******************************************************************************
 * {COPYRIGHT-TOP} *** * IBM Confidential * OCO Source Materials 5725-E59
 * 
 * (c) Copyright IBM Corp. 2012 All Rights Reserved.
 * 
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 ******************************************************************************/
define(
        [ "dojo/dom", "dojo/dom-class", "dojo/dom-construct", "dojo/dom-style",
                "dojo/on", "dojo/aspect", "dojo/_base/lang", "dojo/_base/xhr",
                "dojo/_base/array", "dojo/_base/unload", "dojo/date/locale",
                "dojo/date/stamp", "dijit/layout/BorderContainer",
                "dijit/layout/ContentPane", "wodm/decisions/LoginForm",
                "wodm/decisions/LoginFormDialog", "dijit/Toolbar",
                "dijit/form/Button", "dojo/data/ItemFileWriteStore",
                "dojox/grid/DataGrid", "dijit/tree/ForestStoreModel",
                "dojox/grid/TreeGrid",
                "dojo/i18n!wodm/decisions/nls/manageDecisionsController",
                "ibm/ccd/dialogs/MessageDialog",
                "ibm/ccd/dialogs/CommonDialog", "dijit/Dialog",
                "wodm/decisions/AddRuleDialog", "wodm/decisions/ErrorWidget",
                "wodm/decisions/EditRuleDialog" ],
        function(dom, domClass, domConstruct, domStyle, on, aspect, lang, xhr,
                baseArray, baseUnload, dateLocale, dateStamp, BorderContainer,
                ContentPane, LoginForm, LoginFormDialog, Toolbar, Button,
                ItemFileWriteStore, DataGrid, ForestStoreModel, TreeGrid,
                i18nResources, MessageDialog, CommonDialog, Dialog,
                AddRuleDialog, ErrorWidget, EditRuleDialog) {

            var controller = {

                /**
                 * URLS for various xhr requests/navigation links for the WODM
                 * Manage Decisions UI.
                 */
                _urls : {

                    GET_DECISIONS_AND_RULES : "wodmGetDecisionsAndRules.wpc",
                    VIEW_RULE : "wodmViewRule.wpc",
                    DELETE_RULE : "wodmDeleteRule.wpc",
                    SAVE_RULE : "wodmSaveRule.wpc"
                },

                /**
                 * Object containing configuration properties specific to the
                 * WODM module.
                 */
                _config : dojo.config.wodm,

                /**
                 * Object containing reference to the Border Container widget.
                 */
                _borderContainer : null,

                /**
                 * Object containing reference to the Toolbar Widget.
                 */
                _toolbar : null,

                /**
                 * Object containing reference to the Decisions Pane.
                 */
                _decisionsPane : null,

                /**
                 * Object containing reference to the Rule View Pane.
                 */
                _ruleViewPane : null,

                /**
                 * Dojo data store backing the Decisions Grid.
                 */
                _decisionsStore : null,

                /**
                 * Dojo data grid displaying decision types and rules within
                 * them.
                 */
                _decisionsGrid : null,

                /**
                 * Login form widget to prompt user for login.
                 */
                _loginForm : null,

                /**
                 * Login form dialog to prompt user for login. This is used to
                 * display the login form widget in a dialog.
                 */
                _loginFormDialog : null,

                /**
                 * Array of connection handles. Used to store dojo connections
                 * that can be disconnected on unload.
                 */
                _connectionHandles : [],

                /**
                 * Object containing properties for buttons that will be added
                 * to the Manage Decisions UI Button Toolbar.
                 */
                _buttonPropsMap : null,

                /**
                 * Object containing reference to the AddRuleDialog Widget.
                 */
                _addRuleDialog : null,

                /**
                 * Object containing reference to the EditRuleDialog Widget.
                 */
                _editRuleDialog : null,

                /**
                 * Initializes the WODM Manage Decisions Module and constructs
                 * the UI. This is the entry point for manageDecisionsController
                 * object.
                 */
                init : function() {

                    baseUnload.addOnUnload(lang.hitch(this, this._onUnload));
                    this._createUI();

                },

                /**
                 * Cleanup method invoked on unload of containing iframe.
                 */
                _onUnload : function() {
                    var self = this;
                    baseArray.forEach(this._connectionHandles,
                            function(handle) {
                                handle.remove();
                            });
                    // Workaround for IE9 timing issue during logout tear down
                    setTimeout(function() {
                        if (self._decisionsGrid) {
                            self._decisionsGrid.destroyRecursive();
                         }

                         if (self._borderContainer) {
                            self._borderContainer.destroyRecursive();
                         }

                         if (self._addRuleDialog) {
                             self._addRuleDialog.destroyRecursive();
                         }

                         self._decisionsGrid = null;
                         self._borderContainer = null;
                         self._decisionsPane = null;
                         self._ruleViewPane = null;
                         self._toolbar = null;
                    }, 1000);
                },

                /**
                 * Creates the UI for the WODM Manage Decisions Page.
                 */
                _createUI : function() {

                    this._createBorderContainer();

                    if (!this._verifyAccess(true)) {
                        this._createLoginForm(null, lang.hitch(this,
                                this._createDecisionsUI));

                    } else {
                        this._createDecisionsUI();
                    }

                    this._borderContainer.startup();

                },

                /**
                 * Creates a border container which is the Layout container for
                 * the entire WODM Manage Decisions UI.
                 */
                _createBorderContainer : function() {

                    var borderContainerDiv, borderContainer;
                    borderContainerDiv = dom.byId("borderContainer");
                    borderContainer = new BorderContainer({
                        gutters : false
                    }, borderContainerDiv);

                    this._borderContainer = borderContainer;
                    domClass.add(this._borderContainer.domNode,
                            "wodmBorderContainer");
                },

                /**
                 * Creates the Manage Decisions UI. This is only invoked after
                 * the user has already logged in or does not need to log in due
                 * to shared session usage being view or edit.
                 */
                _createDecisionsUI : function() {

                    this._createToolbar();
                    this._createDecisionsPane();
                    this._createRuleViewPane();
                    this._enableDisableScreenControls();

                },

                /**
                 * Enables or disables controls on the screen based on the state
                 * of the screen.
                 */
                _enableDisableScreenControls : function() {

                    var selectedEntry, selectedEntries, buttons, self;

                    selectedEntries = this._decisionsGrid.selection
                            .getSelected();
                    if (selectedEntries.length > 0) {
                        selectedEntry = selectedEntries[0];
                    }

                    buttons = this._toolbar.getChildren();
                    self = this;
                    baseArray.forEach(buttons, function(button) {
                        var buttonId, shouldEnable;

                        buttonId = button.get("id");
                        shouldEnable = self._shouldEnableToolbarBtn(buttonId,
                                selectedEntry);
                        self._enableDisableToolbarBtn(shouldEnable, button);

                    });

                },

                /**
                 * Returns true if the given entry is a rule, false otherwise.
                 * This check is based on the fact that the decisionType
                 * attribute is only available in rule entries containing a
                 * reference to the decisionType the rule entry belongs to.
                 * 
                 * 
                 * @param entry
                 *            {dojo.data.Item} a dojo store item representing an
                 *            entry
                 * 
                 * @return true if the given entry is a rule, false otherwise
                 */
                _isEntryTypeRule : function(entry) {

                    var decisionType = this._decisionsStore.getValue(entry,
                            "decisionType");

                    return decisionType ? true : false;

                },

                /**
                 * Returns true of the given button id should be enabled, false
                 * otherwise. The decision to enable or disable a button is
                 * taken based on the current state of the UI and the access
                 * level for the given entry.
                 * 
                 * @param buttonId
                 *            the button id to evaluate
                 * @param selectedEntry
                 *            {dojo.data.Item} a dojo store item representing an
                 *            entry
                 * 
                 * 
                 * @return true of the given button id should be enabled, false
                 *         otherwise
                 */
                _shouldEnableToolbarBtn : function(buttonId, selectedEntry) {

                    var accessLevel, isRule, decisionTypeId, accessLevel, isCommonRule;
                    if (!selectedEntry) {
                        return false;
                    }

                    isRule = this._isEntryTypeRule(selectedEntry);
                    isCommonRule = isRule
                            && this._decisionsStore.getValue(selectedEntry,
                                    "isCommon");
                    accessLevel = this._decisionsStore.getValue(selectedEntry,
                            "access");

                    if ((buttonId === "wodmtoolbar.editRule" || buttonId === "wodmtoolbar.deleteRule")
                            && (!isRule || accessLevel !== "edit" || isCommonRule)) {
                        return false;
                    }

                    if (buttonId === "wodmtoolbar.addRule"
                            && (accessLevel !== "edit")) {
                        return false;
                    }

                    return true;

                },

                /**
                 * Enable or disables a button.
                 * 
                 * @param enable
                 *            {boolean} indicates whether to enable or disable
                 *            the button
                 * @param button
                 *            {dijit.form.Button} the button to enable or
                 *            disable
                 */
                _enableDisableToolbarBtn : function(enable, button) {

                    var isDisabled, disabledAttrValue;
                    disabledAttrValue = !enable;
                    isDisabled = button.get("disabled");
                    if (isDisabled !== disabledAttrValue) {
                        button.set("disabled", disabledAttrValue);
                    }

                },

                /**
                 * Initializes the properties of different buttons used in the
                 * Button Toolbar for the Manage Decisions Screen. This includes
                 * definition as well as handlers for each button. The
                 * properties will be available in the _buttonPropsMap after
                 * initialization.
                 */
                _initializeButtonPropsMap : function() {

                    this._buttonPropsMap = {

                        "wodmtoolbar.editRule" : {

                            definition : {

                                id : "wodmtoolbar.editRule",
                                label : i18nResources.TOOLBAR_EDIT_RULE_BUTTON_LABEL,
                                title : i18nResources.TOOLBAR_EDIT_RULE_BUTTON_TOOLTIP
                            },
                            handlers : {

                                click : lang.hitch(this,
                                        this._onEditRuleButtonClick)

                            }

                        },

                        "wodmtoolbar.addRule" : {

                            definition : {

                                id : "wodmtoolbar.addRule",
                                label : i18nResources.TOOLBAR_ADD_RULE_BUTTON_LABEL,
                                title : i18nResources.TOOLBAR_ADD_RULE_BUTTON_TOOLTIP
                            },
                            handlers : {

                                click : lang.hitch(this,
                                        this._onAddRuleButtonClick)

                            }

                        },

                        "wodmtoolbar.deleteRule" : {

                            definition : {

                                id : "wodmtoolbar.deleteRule",
                                label : i18nResources.TOOLBAR_DELETE_RULE_BUTTON_LABEL,
                                title : i18nResources.TOOLBAR_DELETE_RULE_BUTTON_TOOLTIP
                            },
                            handlers : {

                                click : lang.hitch(this,
                                        this._onDeleteRuleButtonClick)

                            }

                        }

                    }

                },

                /**
                 * Creates the Button Toolbar for the Manage Decisions Page.
                 * Adds the toolbar as the top region to the border container.
                 */
                _createToolbar : function() {

                    var toolbar, editRuleButtonProps, addRuleButtonProps, deleteRuleButtonProps;
                    if (!this._buttonPropsMap) {

                        this._initializeButtonPropsMap();

                    }

                    toolbar = new Toolbar({

                        region : "top",
                        height : "200px"

                    });

                    this._toolbar = toolbar;

                    editRuleButtonProps = this._buttonPropsMap["wodmtoolbar.editRule"];
                    addRuleButtonProps = this._buttonPropsMap["wodmtoolbar.addRule"];
                    deleteRuleButtonProps = this._buttonPropsMap["wodmtoolbar.deleteRule"];
                    toolbar.addChild(this._createButton(editRuleButtonProps));
                    toolbar.addChild(this._createButton(addRuleButtonProps));
                    toolbar.addChild(this._createButton(deleteRuleButtonProps));
                    this._borderContainer.addChild(toolbar);

                },

                /**
                 * Creates and returns a new dijit.form.Button object based on
                 * the given properties object.
                 * 
                 * @param buttonProps
                 *            {Object} an object containing properties of the
                 *            button to be created. This includes definition as
                 *            well as handlers.
                 * 
                 * @return {dijit.form.Button} a new dijit.form.Button object
                 *         based on the given properties object
                 */
                _createButton : function(buttonProps) {

                    var button, connectionHandle;
                    button = new Button(buttonProps.definition);
                    connectionHandle = on(button, "click",
                            buttonProps.handlers.click);
                    this._connectionHandles.push(connectionHandle);
                    return button;

                },

                /**
                 * Creates the Rule View Pane and adds it as the bottom region
                 * to the border container. The Rule View Pane displays details
                 * about the currently selected Rule row in the Decisions Grid.
                 * If no Rule row is selected, it will display a Select Rule
                 * Message.
                 */
                _createRuleViewPane : function() {

                    var pane = new ContentPane({
                        id : 'ruleViewPane',
                        region : 'bottom',
                        style : 'height:200px;',
                        splitter : true
                    }, domConstruct.create('div'));

                    this._borderContainer.addChild(pane);
                    this._ruleViewPane = pane;
                    this._displaySelectRuleInRuleViewPane();

                },

                /**
                 * Displays the Select View Message in the Rule View Pane
                 * Content Pane.
                 */
                _displaySelectRuleInRuleViewPane : function() {

                    var selectViewDiv, selectViewTextDiv;

                    selectViewDiv = domConstruct.create("div", {

                        id : "selectViewDiv"
                    });

                    selectViewTextDiv = domConstruct
                            .create(
                                    "div",
                                    {

                                        id : "selectViewTextDiv",
                                        innerHTML : i18nResources.RULE_VIEW_PANE_SELECT_RULE_MSG

                                    });

                    selectViewDiv.appendChild(selectViewTextDiv);
                    this._ruleViewPane.set("content", selectViewDiv);
                },

                /**
                 * Creates the Decisions Pane (i.e Pane containing the Decisions
                 * Grid) and adds it as the center region to the border
                 * container. This method invokes methods to setup the Decisions
                 * Store and the Decisions Grid too.
                 * 
                 */
                _createDecisionsPane : function() {

                    this._decisionsPane = new ContentPane({
                        id : 'decisionsPane',
                        region : 'center',
                        splitter : true
                    }, domConstruct.create('div'));

                    this._setupDecisionsStore();
                    // keep the claro styling for the grid.
                    domClass.add(this._decisionsPane.domNode, "claro");
                    this._setupDecisionsGrid();
                    this._borderContainer.addChild(this._decisionsPane);

                },

                /**
                 * Sets up the Decisions Store which is the data store backing
                 * the Decisions Grid shown in the Decisions Pane.
                 */
                _setupDecisionsStore : function() {
                    var typeMap, url;
                    var _selfCtrl = this;
                    typeMap = {
                        "Date" : {
                            type : Date,
                            deserialize : function(value) {
                                return dateStamp.fromISOString(value);
                            }
                        }
                    };

                    var storeData = new Object();
                    storeData.identifier = "id";
                    storeData.items = new Array();

                    url = this._urls.GET_DECISIONS_AND_RULES + "?"
                            + this._getCommonQueryParamsString();

                dojo.xhrGet( {
                            url : url,
                            handleAs : "json",
                            sync : true,
                            load : function(data) {
                                if(data.error)
                                {
                                    _selfCtrl._handleResponseError(data.error);
                                }else
                                {
                                    storeData = data;
                                }
                            },
                            error : function(error) {
                                _selfCtrl._handleResponseError(error);
                            }
                        });

                        this._decisionsStore = new ItemFileWriteStore({
                        data : storeData,
                        typeMap : typeMap
                    });

                },

                /**
                 * Creates and initializes the Decisions Grid and adds it to the
                 * Decisions Pane content pane.
                 */
                _setupDecisionsGrid : function() {

                    var layout, grid, gridDiv;
                    var layout = [
                            {
                                name : i18nResources.DECISIONS_PANE_GRID_NAME_COLUMN_LABEL,
                                field : 'name',
                                width : '30%',
                                formatter : lang.hitch(this,
                                        this._decisionNameFormatter)
                            },
                            {
                                name : i18nResources.DECISIONS_PANE_GRID_COMMON_COLUMN_LABEL,
                                field : 'isCommon',
                                width : '6%',
                                formatter : lang.hitch(this,
                                        this._commonFormatter)
                            },
                            {
                                name : i18nResources.DECISIONS_PANE_GRID_TYPE_COLUMN_LABEL,
                                field : 'type',
                                width : '15%',
                                formatter : lang.hitch(this,
                                        this._ruleTypeFormatter)
                            },
                            {
                                name : i18nResources.DECISIONS_PANE_GRID_PRIORITY_COLUMN_LABEL,
                                field : 'priority',
                                width : '10%',
                                formatter : lang.hitch(this,
                                        this._defaultFormatter)
                            },
                            {
                                name : i18nResources.DECISIONS_PANE_GRID_LAST_CHANGED_BY_COLUMN_LABEL,
                                field : 'lastChangedBy',
                                width : '20%',
                                formatter : lang.hitch(this,
                                        this._defaultFormatter)
                            },
                            {
                                name : i18nResources.DECISIONS_PANE_GRID_LAST_CHANGED_ON_COLUMN_LABEL,
                                field : 'lastChangedOn',
                                width : 'auto',
                                formatter : lang.hitch(this,
                                        this._dateFormatter)
                            }

                    ];

                    gridDiv = domConstruct.create("div");

                    var treeModel = new ForestStoreModel({
                        store : this._decisionsStore,
                        query : {
                            id : '*'
                        },

                        childrenAttrs : [ 'rules' ]
                    });
                    var grid = new TreeGrid(
                            {
                                treeModel : treeModel,
                                structure : layout,
                                rowSelector : true,
                                defaultOpen : false,
                                keepSelection : true,
                                selectionMode : 'single',
                                noDataMessage : i18nResources.DECISIONS_PANE_GRID_NO_DATA_MSG
                            }, gridDiv);

                    aspect.after(grid, "onSelectionChanged", lang.hitch(this,
                            this._onGridSelectionChanged));

                    grid.canSort = function(sortInfo) {

                        if (sortInfo !== 1 && sortInfo !== -1) {
                            return false;
                        } else {
                            return (!grid._isLoading);
                        }
                    };

                    grid.startup();

                    this._decisionsGrid = grid;
                    dojo.addClass(grid.domNode, "decisionsGrid");
                    this._decisionsPane.set("content", grid.domNode);

                },

                /**
                 * Returns an html img string if the given value is true
                 * indicating the entry is a common rule, an empty string
                 * otherwise. This method is specified as the formatter for the
                 * common column in the decisions grid.
                 * 
                 * @param value
                 *            {String} value being formatted
                 * @param rowIndex
                 *            {int} the index of the row being formatted
                 * 
                 * @return an html img string if the given value is true
                 *         indicating the entry is a common rule, an empty
                 *         string otherwise
                 * 
                 */
                _commonFormatter : function(value, rowIndex) {

                    if (value) {
                        return "<img class='commonRuleImage' src='/user/wodm/images/checkmark.gif' title='"
                                + i18nResources.DECISIONS_PANE_GRID_COMMON_COLUMN_IMAGE_TOOLTIP
                                + "'></img>";

                    }

                    return "";

                },

                /**
                 * Returns the localized string for rule type given by value if
                 * it is not null or undefined, otherwise returns an empty
                 * string. This method is specified as the formatter for the
                 * Type column in the manage decisions grid.
                 * 
                 * @param value
                 *            {String} wodm rule type as a string
                 * @param rowIndex
                 *            {int} the index of the row being formatted
                 * 
                 * @return the localized string for rule type given by value if
                 *         it is not null or undefined, otherwise returns an
                 *         empty string
                 */
                _ruleTypeFormatter : function(value, rowIndex) {
                    if (value === null || value === undefined) {
                        return "";
                    } else if (value === "brm.ActionRule") {
                        return i18nResources.RULE_TYPE_ACTION_RULE;
                    } else if (value === "brm.DecisionTable") {
                        return i18nResources.RULE_TYPE_DECISION_TABLE;
                    } else {
                        return "";
                    }

                },

                /**
                 * Returns the value as is if it is not null or undefined,
                 * otherwise returns an empty string. This method is specified
                 * as the default formatter for columns on the decisions grid on
                 * which no other formatter has been applied.
                 * 
                 * @param value
                 *            {String} value being formatted
                 * @param rowIndex
                 *            {int} the index of the row being formatted
                 * 
                 * @return the value as is if it is not null or undefined,
                 *         otherwise returns an empty string
                 */
                _defaultFormatter : function(value, rowIndex) {
                    if (value === null || value === undefined) {
                        return "";
                    } else {
                        return value;
                    }

                },

                /**
                 * 
                 * Returns a date string formatted according to the current
                 * locale from the date string given in the value parameter. If
                 * the value parameter is null or undefined, an empty string is
                 * returned. This method is specified as the formatter for the
                 * Last Changed On column in the manage decisions grid.
                 * 
                 * 
                 * @param value
                 *            {Date} the javascript Date object
                 * @param rowIndex
                 *            the index of the row being formatted
                 * @returns {String} a date string formatted according to the
                 *          current locale from the date string given in the
                 *          value parameter
                 */
                _dateFormatter : function(value, rowIndex) {

                    if (value === null || value === undefined) {
                        return "";
                    }

                    return dateLocale.format(value);
                },

                /**
                 * Formatter function for the Name column in the grid.
                 * 
                 * @param value
                 *            {String} the original value for the Name column
                 * @param rowIndex
                 *            the index of the row being formatted
                 * @param rowAndChildIndex
                 *            {Array} array containing row index and child index
                 *            if applicable
                 * 
                 * 
                 * @returns {String} the original value if invoked for a rule, a
                 *          formatted html string if invoked for a decision type
                 */
                _decisionNameFormatter : function(value, rowIndex,
                        rowAndChildIndex) {

                    var decisionTypeNameSpan;

                    // called for the child rather than the parent.
                    if (rowAndChildIndex.length > 1) {
                        return value;
                    }
                    return "<span class='decisionTypeName'>" + value
                            + "</span>";

                },

                /**
                 * Returns an object containing information extracted from the
                 * dojo data store item given. Extracting this out as many
                 * requests need to do this.
                 * 
                 * @param rule
                 *            {dojo.data.item} dojo data store item representing
                 *            a rule
                 * 
                 * @returns an object containing information extracted from the
                 *          dojo data store item given
                 * 
                 */
                _getRuleInfo : function(rule) {

                    var ruleInfo = {};
                    ruleInfo.ruleName = this._decisionsStore.getValue(rule,
                            "name");
                    ruleInfo.ruleProjectName = this._decisionsStore.getValue(
                            rule, "ruleProjectName");
                    ruleInfo.decisionType = this._decisionsStore.getValue(rule,
                            "decisionType");
                    ruleInfo.isCommon = this._decisionsStore.getValue(rule,
                    "isCommon");

                    return ruleInfo;
                },

                /**
                 * Returns an object containing information extracted from the
                 * dojo data store item given. Extracting this out as many
                 * requests need to do this.
                 * 
                 * @param decisionType
                 *            {dojo.data.item} dojo data store item representing
                 *            a decisionType
                 * 
                 * @returns an object containing information extracted from the
                 *          dojo data store item given
                 * 
                 */
                _getDecisionTypeInfo : function(decisionType) {

                    var decisionInfo = {};
                    decisionInfo.decisionType = this._decisionsStore.getValue(
                            decisionType, "id");
                    decisionInfo.decisionTypeName = this._decisionsStore
                            .getValue(decisionType, "name");
                    decisionInfo.ruleProjectName = this._decisionsStore
                            .getValue(decisionType, "ruleProjectName");

                    return decisionInfo;

                },

                /**
                 * Displays the loading ICON in the reportDetailsPane Content
                 * Pane.
                 */
                _displayLoaderInRuleViewPane : function() {

                    this._ruleViewPane.set("content",
                            "<div class='loader'></div>");
                },

                /**
                 * Event handler for selection change in the decisions grid in
                 * the decisionsPane Content Pane.
                 */
                _onGridSelectionChanged : function(e) {

                    var selectedEntry, ruleProjectName, ruleName;

                    selectedEntry = this._decisionsGrid.selection.getSelected()[0];

                    if (!selectedEntry) {
                        this._displaySelectRuleInRuleViewPane();
                        this._enableDisableScreenControls();
                        return;
                    }
                    if (this._isEntryTypeRule(selectedEntry)) {
                        this._displayLoaderInRuleViewPane();
                        this._sendViewRuleRequest(selectedEntry);

                    } else {

                        this._displaySelectRuleInRuleViewPane();
                    }

                    this._enableDisableScreenControls();

                },

                /**
                 * Creates the LoginForm widget if not already created and
                 * renders it in a pane or in a dialog according to the given
                 * dialog parameter.
                 * 
                 * @param displayInDialog
                 *            {boolean} indicates whether to display the
                 *            LoginForm Widget in a dialog or not
                 * @param onLoginCallBack
                 *            {Function} call back function which will be
                 *            invoked when a user successfully logs in using the
                 *            LoginForm Widget.
                 * 
                 */
                _createLoginForm : function(displayInDialog, onLoginCallback) {

                    var loginContentPane;
                    if (!this._loginForm) {
                        this._loginForm = new LoginForm();

                        aspect.after(this._loginForm, "onLoginSuccess", lang
                                .hitch(this, this._onLoginSuccess,
                                        onLoginCallback));

                    }

                    if (!displayInDialog) {
                        loginContentPane = new ContentPane({

                            region : 'center'
                        });

                        loginContentPane
                                .set("content", this._loginForm.domNode);
                        this._borderContainer.addChild(loginContentPane);

                    } else {

                        if (!this._loginFormDialog) {
                            this._loginFormDialog = new LoginFormDialog({
                                loginForm : this._loginForm
                            });
                        }

                        this._loginFormDialog.show();
                    }

                },

                /**
                 * Handler for the onClick event of the Edit Rule Button in the
                 * toolbar.
                 */
                _onEditRuleButtonClick : function() {

                    var rule;
                    rule = this._decisionsGrid.selection.getSelected()[0];
                    if (!this._isEntryTypeRule(rule)) {

                        return;
                    }

                    if (!this._verifyAccess(false)) {
                        this._createLoginForm(true, lang.hitch(this,
                                this._launchEditRuleDialog, rule));

                    } else {

                        this._launchEditRuleDialog(rule);
                    }

                },

                /**
                 * Handler for the onClick event of the Add Rule Button in the
                 * toolbar.
                 */
                _onAddRuleButtonClick : function() {

                    var selectedEntry;
                    selectedEntry = this._decisionsGrid.selection.getSelected()[0];
                    if (!selectedEntry) {
                        return;
                    }

                    if (!this._verifyAccess(false)) {
                        this._createLoginForm(true, lang.hitch(this,
                                this._addRule, selectedEntry));

                    } else {

                        this._addRule(selectedEntry);
                    }

                },

                /**
                 * Handler for the onClick event of the Delete Rule Button in
                 * the toolbar.
                 */
                _onDeleteRuleButtonClick : function() {

                    var rule, confirmMessage, confirmed;

                    confirmMessage = i18nResources.CONFIRM_TO_DELETE_RULE;
                    confirmed = confirm(confirmMessage);
                    if (!confirmed) {
                        return;
                    }

                    rule = this._decisionsGrid.selection.getSelected()[0];
                    if (!rule) {
                        return;
                    }

                    if (!this._verifyAccess(false)) {
                        this._createLoginForm(true, lang.hitch(this,
                                this._sendDeleteRuleRequest, rule));

                    } else {

                        this._sendDeleteRuleRequest(rule);
                    }

                },

                /**
                 * Invokes the launchAddRuleDialog passing it the decision type
                 * to add the new rule to.
                 * 
                 * @param selectedEntry
                 *            {dojo.data.item} dojo data store item representing
                 *            a rule or decision type in the decisions grid
                 */
                _addRule : function(selectedEntry) {

                    if (this._isEntryTypeRule(selectedEntry)) {

                        this._decisionsStore.fetchItemByIdentity({

                            identity : selectedEntry.decisionType,
                            onItem : function(item) {

                                this._launchAddRuleDialog(item);

                            },
                            scope : this

                        })

                    } else {

                        this._launchAddRuleDialog(selectedEntry);
                    }

                },

                /**
                 * Launches the AddDialogWidget if it exists, otherwise lauches
                 * one after creating it.
                 * 
                 * @param decisionType
                 *            {dojo.data.item} dojo data store item representing
                 *            a decision type
                 */
                _launchAddRuleDialog : function(decisionType) {

                    var decisionTypeInfoObj = this
                            ._getDecisionTypeInfo(decisionType);
                    if (!this._addRuleDialog) {

                        this._addRuleDialog = new AddRuleDialog({

                            decisionTypeInfo : decisionTypeInfoObj
                        });

                        aspect.after(this._addRuleDialog, "onSuccessfulSave",
                                lang.hitch(this, this._onAddRuleSuccess), true);
                    } else {

                        this._addRuleDialog.set("decisionTypeInfo",
                                decisionTypeInfoObj);

                    }

                    this._addRuleDialog.show();

                },

                /**
                 * Launches the EditRuleDialog if it exists, otherwise launches
                 * one after creating it.
                 * 
                 * @param rule
                 *            {dojo.data.item} dojo data store item representing
                 *            a rule
                 */
                _launchEditRuleDialog : function(rule) {

                    var ruleProjectName, ruleName;
                    ruleProjectName = this._decisionsStore.getValue(rule,
                            "ruleProjectName");
                    ruleName = this._decisionsStore.getValue(rule, "name");

                    if (!this._editRuleDialog) {

                        this._editRuleDialog = new EditRuleDialog({
                            ruleName : ruleName,
                            ruleProjectName : ruleProjectName
                        });

                        aspect.after(this._editRuleDialog, "onSuccessfulSave",
                        		lang.hitch(this, this._onSaveRuleSuccess, rule), true);

                        aspect.after(this._editRuleDialog, "onSuccessfulCancel",
                        		lang.hitch(this, this._onCancelRuleSuccess), true);

                    } else {

                        this._editRuleDialog.set("ruleName", ruleName);
                        this._editRuleDialog.set("ruleProjectName",
                                ruleProjectName);
                    }

                    this._editRuleDialog.show();

                },

                /**
                 * Call back which is invoked by the AddRuleDialog Widget on
                 * successfully creating a new rule. This method creates a new
                 * dojo data store item to insert the newly created rule in the
                 * decisions store.
                 * 
                 * @param rule
                 *            {Object} a json object representing the newly
                 *            created rule on the server
                 * @param editImmediately
                 *            {boolean} true if the rule editor should be
                 *            immediately launched after the creation of a new
                 *            rule, false otherwise
                 */
                _onAddRuleSuccess : function(rule, editImmediately) {

                    this._addRuleDialog.hide();
                    this._decisionsStore.fetchItemByIdentity({

                        identity : rule.decisionType,
                        onItem : function(item) {

                            var parentInfoObj, newRule;

                            parentInfoObj = {

                                parent : item,
                                attribute : "rules"

                            }
                            rule.lastChangedOn = dateStamp
                                    .fromISOString(rule.lastChangedOn._value);
                            newRule = this._decisionsStore.newItem(rule,
                                    parentInfoObj);

                            if (editImmediately) {
                                this._launchEditRuleDialog(newRule);
                            }

                        },
                        scope : this

                    });

                },

                /**
                 * Call back which is invoked by the EditRuleDialog Widget on
                 * successfully saving a rule. This method updates the dojo
                 * store item edited to include changes retured by the server in
                 * the updatedRuleObj parameter.
                 * 
                 * @param rule
                 *            {dojo.data.item} The dojo store item representing
                 *            the rule that was saved
                 * @param updatedRuleObj
                 *            {Object} json object containing updated properties
                 *            of the edited rule
                 * 
                 */
                _onSaveRuleSuccess : function(rule, updatedRuleObj) {

                    var ruleProjectName, ruleName;
                    this._decisionsStore.setValue(rule, "lastChangedBy",
                            updatedRuleObj.lastChangedBy);
                    this._decisionsStore
                            .setValue(
                                    rule,
                                    "lastChangedOn",
                                    dateStamp
                                            .fromISOString(updatedRuleObj.lastChangedOn._value));

                    this._editRuleDialog.hide();

                    this._onGridSelectionChanged();

                },

                _onCancelRuleSuccess : function() {
                    this._onGridSelectionChanged();
                },
                
                /**
                 * Sets the isUserLoggedIn property to true in the WODM config
                 * object. Also closes and destroys the LoginForm and
                 * LoginFormDialog (if relevant), and invokes the call back
                 * originally provided by the caller to the _createLoginForm
                 * method.
                 * 
                 * @param onLoginCallback
                 *            {Function} call back function orginally provided
                 *            by the caller to the _createLoginForm method
                 * 
                 */
                _onLoginSuccess : function(onLoginCallback) {

                    this._config.isUserLoggedIn = true;

                    if (this._loginFormDialog) {
                        this._loginFormDialog.hide();
                        this._loginFormDialog.destroyRecursive();
                        this._loginForm.destroyRecursive();
                    } else {
                        this._borderContainer.removeChild(this._loginForm);
                        this._loginForm.destroyRecursive();
                    }

                    this._loginFormDialog = null;
                    this._loginForm = null;
                    if (onLoginCallback) {
                        onLoginCallback();
                    }

                },

                _getCommonQueryParamsString : function() {

                    var paramsStr = "";
                    paramsStr += "entryId=" + this._config.entryId;
                    paramsStr += "&entryPk=" + this._config.entryPk;
                    paramsStr += "&containerName=" + this._config.containerName;
                    paramsStr += "&containerType=" + this._config.containerType;
                    if (this._config.stepId) {
                        paramsStr += "&stepId=" + this._config.stepId;
                    }
                    return paramsStr;

                },

                /**
                 * Returns true if the user has access, false otherwise.
                 * 
                 * @param viewOnly
                 *            {boolean} true if only view access needs to be
                 *            verified, false otherwise
                 * @returns true if the user has access, false otherwise
                 */
                _verifyAccess : function(viewOnly) {

                    var hasAccess = false;
                    var sharedSessionUsage = this._config.sharedSessionUsage;
                    if (viewOnly) {

                        if (this._config.isUserLoggedIn
                                || sharedSessionUsage === "VIEW"
                                || sharedSessionUsage === "EDIT") {
                            hasAccess = true;
                        }

                    } else {

                        if (this._config.isUserLoggedIn
                                || sharedSessionUsage === "EDIT") {
                            hasAccess = true;
                        }

                    }

                    return hasAccess;

                },

                /**
                 * A Generic error handler for XHR requests.
                 * 
                 * @param error
                 *            {Object/String} contains error details.
                 */
                _handleResponseError : function(error) {

                    var messageDetails, errorDialog;
                    // this._onAfterLoad(true);
                    messageDetails = "";
                    if (dojo.isString(error)) {
                        messageDetails = error;
                    } else if (dojo.isObject(error)) {

                        messageDetails = error.message || error.errorMessage
                                || error.errorMessages;

                    }
                    errorDialog = new MessageDialog({
                        messageMode : "error",
                        messageDetails : messageDetails,
                        showOkButton : false,
                        style : "width: 350px;"
                    });

                    errorDialog.show();

                    aspect.after(errorDialog, "onCancel", function() {
                        errorDialog.destroy();
                    });

                },

                /**
                 * Sends a request to the server to delete the given rule.
                 * 
                 * @param {dojo.data.item}
                 *            dojo data store item representing a rule
                 */
                _sendDeleteRuleRequest : function(rule) {

                    var responseHandler = lang.hitch(this,
                            this._handleDeleteRuleRequestResponse);
                    this._sendRuleRequest(rule, this._urls.DELETE_RULE,
                            responseHandler, responseHandler);

                },

                /**
                 * Response handler for the delete rule xhr request.
                 * 
                 * @param response
                 *            {Object} response received from the server for the
                 *            view rule request
                 * @param rule
                 *            {dojo.data.item} dojo data store item for which
                 *            the delete request was issues
                 */
                _handleDeleteRuleRequestResponse : function(response, rule) {

                    if (response.error) {

                        this._handleResponseError(response.error);
                        return;
                    }

                    this._decisionsGrid.selection.deselectAll();
                    this._decisionsStore.deleteItem(rule);
                    this._decisionsStore.save();

                },

                /**
                 * Sends a request to the server to retrieve details about the
                 * given rule.
                 * 
                 * @param {dojo.data.item}
                 *            dojo data store item representing a rule
                 */
                _sendViewRuleRequest : function(rule) {

                    var responseHandler = lang.hitch(this,
                            this._handleViewRuleRequestResponse);
                    this._sendRuleRequest(rule, this._urls.VIEW_RULE,
                            responseHandler, responseHandler);

                },

                /**
                 * Response handler for the view rule xhr request.
                 * 
                 * @param response
                 *            {Object} response received from the server for the
                 *            view rule request
                 */
                _handleViewRuleRequestResponse : function(response) {

                    if (response.error) {
                        this._displaySelectRuleInRuleViewPane();
                        this._handleResponseError(response.error);
                        return;
                    }

                    var html = "<div id=\"viewRuleDiv\">" + response.ruleHtml
                            + "</div>";
                    this._ruleViewPane.set("content", html);

                },

                /**
                 * Sends a xhr get request for the given rule. Rule information
                 * is extracted from the given rule dojo store item and mixed in
                 * with the get query params to send along with the request. The
                 * response handlers are passed the response and given rule dojo
                 * data store item.
                 * 
                 * @param rule
                 *            {dojo.data.item} dojo data store item representing
                 *            a rule
                 * @param url
                 *            {String} url for the xhr get request
                 * @param onLoad
                 *            {Function} call back to execute on response
                 * @param onError
                 *            {Function} call back to execute on error
                 * 
                 * 
                 */
                _sendRuleRequest : function(rule, url, onLoad, onError) {
                    var reqObj, ruleInfoObj, content;
                    ruleInfoObj = this._getRuleInfo(rule);
                    content = this._config;
                    lang.mixin(content, ruleInfoObj);

                    var reqObj = {

                        url : url,
                        handleAs : "json",
                        content : content,
                        sync : true,
                        load : function(response) {

                            onLoad(response, rule);

                        },
                        error : function(error) {
                            var errorObj = {
                                error : error
                            }
                            onError(errorObj);
                        }

                    }

                    setTimeout(function() {

                        xhr.get(reqObj);
                    }, 0);

                }

            };

            return controller;

        });